/*
 * Project: dice
 * File name: dice7.c
 * Description: Generate 
 *   
 * author Lucien 'Drurowin' Pullen. Copyright (C) 2010.
 * version $Id: LICENSE.txt,v 2 2010/11/2 14:24:26 lucien Exp $
 *   
 * See The GNU Public License (GPL)
 *
 * 
 * This program is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation; either version 2 of the License, or 
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

// Delete or comment out the items you do not need.
#define COMMENT_STRING         "Lucien Pullen's overpowered PRNG"
#define COMMENT_PROGRAM_NAME   "dice7"
#define COMMENT_VERSION_STRING "7.1.0.0"
#define COMMENT_VERSION_NUMBER 7,1,0,0 /* major, minor, revision, subrevision */
//#define COMMENT_BW_ICON \
	{0b0000000000000000, \
	 0b0000000000000000, \
	 0b0000000000000000, \
	 0b0000000000000000, \
	 0b0000000000000000, \
	 0b0000000000000000, \
	 0b0000000000000000, \
	 0b0000000000000000, \
	 0b0000000000000000, \
	 0b0000000000000000, \
	 0b0000000000000000, \
	 0b0000000000000000, \
	 0b0000000000000000, \
	 0b0000000000000000, \
	 0b0000000000000000, \
	 0b0000000000000000}

#define SAVE_SCREEN
#define RETURN_VALUE

#define USE_TI89

#include <c/tigcclib.h>

// Error messages
char* ERR_ARG	="Argument Error";
char* DET_ARG	="dice7(sides,bonus[,times])";
char* ERR_DMA	="DMA Failure";
char* DET_DMA	="Unable to allocate memory.";
char* COPY	="dice7 Copyright (C) 2010 Lucien 'Drurowin' Pullen";

// Function prototypes
void cleanEStack(void);

// Main Function
void _main(void)
{
	// Get arguments
	ESI argptr = top_estack;
	short sides, bonus, times;
	if(ArgCount() < 2 || ArgCount() > 3) {
		DlgMessage(ERR_ARG, DET_ARG, BT_OK, BT_NONE);
		cleanEStack();
		push_END_TAG();
		push_LIST_TAG();
		return;
	}
	if(GetArgType(argptr) == POSINT_TAG) {
		sides = GetIntArg(argptr);
	} else {
		DlgMessage(ERR_ARG, DET_ARG, BT_OK, BT_NONE);
		cleanEStack();
		push_END_TAG();
		push_LIST_TAG();
		return;
	}
	if(GetArgType(argptr) == POSINT_TAG) {
		bonus = GetIntArg(argptr);
	} else if(GetArgType(argptr) == NEGINT_TAG) {
		bonus = -GetIntArg(argptr);
	} else {
		DlgMessage(ERR_ARG, DET_ARG, BT_OK, BT_NONE);
		cleanEStack();
		push_END_TAG();
		push_LIST_TAG();
		return;
	}
	if(ArgCount() == 3 && GetArgType(argptr) == POSINT_TAG) {
		times = GetIntArg(argptr);
	} else {
		times = 1;
	}

	// Clean up after getting args
	cleanEStack();

	// Make variables we'll use
	short* res;
	if((res = (short*)malloc(times * sizeof(short))) == NULL) {
		DlgMessage(ERR_DMA, DET_DMA, BT_OK, BT_NONE);
		return;
	}
	long sumres = 0;

	// Randomize the random number generator...
	//   Hopefully this will fix the bug when running from the archive
	randomize();

	// Store random numbers to result array and add to sum
	short i;
	for(i = 0; i < times; i++) {
		res[i] = (rand() % sides + 1);
		// Normalize for D&D (no negative dice rolls)
		if(res[i] < 0) res[i] = 0;

		// Add modifier
		res[i] += bonus;

		// Add to sum (aka:: DM's friend)
		sumres += res[i];
	}

	// Return list with results: [sum, x1, x2, ..., xn]
	push_END_TAG();
	for(i = times-1; i >= 0; i--) {
		push_shortint(res[i]);
	}
	push_longint(sumres);
	push_LIST_TAG();

	// Nuke that (potentially) huge malloc
	free(res);
}

/*
 * Clean up the expression stack
 */
void cleanEStack() {
	while(GetArgType(top_estack) != END_TAG)
		top_estack = next_expression_index(top_estack);
	top_estack--;
}
